import { sequelize } from '../modules/sequelize.ts';
import { DataTypes, Model } from 'sequelize';
import { Variable } from '@kevisual/ai-graph';

/**
 * 预设数据，定义了请求的内容和验证器
 */
export type PresetData = {
  // 参数
  validator: {
    [key: string]: any; // 请求的内容的验证器
  };
  data: {
    prompt?: string; // 提前预设值
    inputs: Variable & { operate?: string }[]; // 请求内容的变量和内容
  };
};
export class Prompt extends Model {
  declare id: string;
  declare title: string;
  declare description: string;
  declare presetData: PresetData;
}

Prompt.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4,
    },
    title: {
      type: DataTypes.STRING,
      allowNull: false,
    },
    description: {
      type: DataTypes.TEXT,
      allowNull: true,
    },
    presetData: {
      type: DataTypes.JSON,
    },
    key: {
      type: DataTypes.STRING,
      allowNull: false,
      unique: true,
    },
    // inputVariables: {
    //   type: DataTypes.JSON,
    //   defaultValue: [],
    // },
    // localVariables: {
    //   type: DataTypes.JSON,
    //   defaultValue: [],
    // },
  },
  {
    sequelize, // 传入 Sequelize 实例
    modelName: 'prompt', // 模型名称
    paranoid: true,
  },
);

Prompt.sync({ alter: true, force: false, logging: false }).catch((e) => {
  console.error('Prompt sync error', e);
});
